using System;
using NUnit.Framework;

namespace Smdn.Protocols.Imap4 {
  [TestFixture()]
  public class ImapUriTest {
    [Test()]
    public void TestUriFormatHostOnly()
    {
      //   imap://<iserver>/
      var uri = new ImapUri("imap://localhost/");

      Assert.AreEqual(string.Empty, uri.UserName);
      Assert.AreEqual("localhost", uri.Server);
      Assert.IsNull(uri.Authentication);
      Assert.AreEqual(ImapUriForm.Server, uri.Form);
    }

    [Test()]
    public void TestUriFormatHostWithPort()
    {
      //   imap://<iserver>/
      var uri = new ImapUri("imap://localhost:993/");

      Assert.AreEqual(string.Empty, uri.UserName);
      Assert.AreEqual("localhost:993", uri.Server);
      Assert.AreEqual(993, uri.Port);
      Assert.AreEqual(ImapUriForm.Server, uri.Form);
    }

    [Test()]
    public void TestUriFormatHostUserAuthentication()
    {
      var uri = new ImapUri("imap://user;auth=plain@localhost");

      Assert.AreEqual("user", uri.UserName);
      Assert.AreEqual("user;auth=plain@localhost", uri.Server);
      Assert.AreEqual(ImapAuthenticationMechanism.Plain, uri.Authentication);
      Assert.AreEqual(null, uri.Mailbox);
      Assert.AreEqual(0, uri.UidValidity);
      Assert.AreEqual(0, uri.Uid);
      Assert.AreEqual(null, uri.Section);
      Assert.AreEqual(ImapUriForm.Server, uri.Form);
    }

    [Test()]
    public void TestUriFormatQuery1()
    {
      //   imap://<iserver>/<enc_mailbox>[uidvalidity][?<enc_search>]
      var uri = new ImapUri("imap://;AUTH=*@minbari.org/gray%20council?SUBJECT%20shadows");

      Assert.AreEqual("gray council", uri.Mailbox);
      Assert.AreEqual("?SUBJECT%20shadows", uri.Query);
      Assert.AreEqual(ImapUriForm.SearchMessages, uri.Form);

      Assert.AreEqual("SUBJECT shadows", uri.GetSearchCriteria().ToString());
    }

    [Test()]
    public void TestUriFormatQuery2()
    {
      var uri = new ImapUri("imap://john;AUTH=*@minbari.example.org/babylon5/personel?charset%20UTF-8%20SUBJECT%20%7B14+%7D%0D%0A%D0%98%D0%B2%D0%B0%D0%BD%D0%BE%D0%B2%D0%B0");

      Assert.AreEqual(ImapUriForm.SearchMessages, uri.Form);
      Assert.AreEqual("charset UTF-8 SUBJECT {14+}\r\nИванова", uri.GetSearchCriteria().ToString());
    }

    [Test()]
    public void TestUriFormatMailbox()
    {
      //   imap://<iserver>/<enc_mailbox>[uidvalidity]<iuid>[isection]
      var uri = new ImapUri("imap://psicorp.org/~peter/%E6%97%A5%E6%9C%AC%E8%AA%9E/%E5%8F%B0%E5%8C%97");

      Assert.AreEqual("~peter/日本語/台北", uri.Mailbox);
      Assert.AreEqual(ImapUriForm.ListMessages, uri.Form);
    }

    [Test()]
    public void TestUriFormatUid()
    {
      var uri = new ImapUri("imap://minbari.org/gray-council;UIDVALIDITY=385759045/;UID=20");

      Assert.AreEqual("gray-council", uri.Mailbox);
      Assert.AreEqual(385759045L, uri.UidValidity);
      Assert.AreEqual(20L, uri.Uid);
      Assert.AreEqual(ImapUriForm.FetchMessage, uri.Form);
    }

    [Test()]
    public void TestUriFormatPartial()
    {
      var uri = new ImapUri("imap://minbari.example.org/gray-council;UIDVALIDITY=385759045/;UID=20/;PARTIAL=0.1024");

      Assert.AreEqual("gray-council", uri.Mailbox);
      Assert.AreEqual(385759045L, uri.UidValidity);
      Assert.AreEqual(20L, uri.Uid);
      Assert.AreEqual(null, uri.Section);
      Assert.AreEqual("0.1024", uri.Partial);
      Assert.AreEqual(ImapUriForm.FetchMessage, uri.Form);
    }

    [Test()]
    public void TestUriFormatSection()
    {
      var uri = new ImapUri("imap://;AUTH=KERBEROS_V4@minbari.org/gray-council/;uid=20/;section=1.2");

      Assert.AreEqual(ImapAuthenticationMechanism.KerberosV4, uri.Authentication);
      Assert.AreEqual("1.2", uri.Section);
      Assert.AreEqual(0L, uri.UidValidity);
      Assert.AreEqual(20L, uri.Uid);
      Assert.AreEqual(ImapUriForm.FetchMessage, uri.Form);
    }
  }
}
